#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <misc.h>
#include <sstream.h>
#include "tlmplib.h"
#include "tlmplib.m"

void _F_loadfile__v1::empty(const LOADFILE_INFO &)
{
}
void _F_loadfile__v1::start(const LOADFILE_INFO &)
{
}
void _F_loadfile__v1::end(off_t, const LOADFILE_INFO &)
{
}
void _F_loadfile__v1::missing(const char *fname)
{
	tlmp_error (MSG_U(E_CANTOPEN,"Can't open file %s (%s)\n")
		,fname,strerror(errno));
}

static int loadfile_gen (
	_F_loadfile__v1 &c,
	SSTREAM &ss,
	bool strip,
	const char *fname)
{
	LOADFILE_INFO info;
	info.filename = fname;
	c.priv = NULL;
	int ret = 0;
	char buf[1000];
	off_t lastpos = ss.getoffset();
	while (ss.gets(buf,sizeof(buf)-1)!=NULL){
		if (ret == 0) c.start(info);
		if (strip) strip_end (buf);
		if (c.oneline (buf,lastpos,ret,info)==-1){
			break;
		}
		lastpos = ss.getoffset();
		ret++;
	}
	if (ret == 0){
		c.empty(info);
	}else{
		c.end (ss.getoffset(),info);
	}
	return ret;
}

int loadfile__v1 (_F_loadfile__v1 &c, SSTREAM &ss, bool strip)
{
	return loadfile_gen (c,ss,strip,"(sstream)");
}

int loadfile__v1 (_F_loadfile__v1 &c, const char *fname, bool strip)
{
	int ret = -1;
	if (strcmp(fname,"-")==0){
		SSTREAM_FILE ss(stdin);
		ret = loadfile_gen (c,ss,strip,"(stdin)");
	}else{
		FILE *fin = fopen(fname,"r");
		if (fin == NULL){
			c.missing (fname);
		}else{
			SSTREAM_FILE ss(fin);
			ret = loadfile_gen (c,ss,strip,fname);
			fclose (fin);
		}
	}
	return ret;
}

int loadfile__v1 (_F_loadfile__v1 &c, CONFIG_FILE &fl, bool strip)
{
	int ret = -1;
	FILE_CFG *fin = fl.fopen("r");
	SSTRING path (fl.getpath());
	if (fin == NULL){
		if (!fl.is_optional()) c.missing (path.get());
	}else{
		SSTREAM_FILE_CFG ss(fin);
		ret = loadfile_gen (c,ss,strip,path.get());
		fclose (fin);
	}
	return ret;
}


void _F_savefile::start(FILE *, const char *){}
void _F_savefile::end(FILE *, const char *){}
void _F_savefile::openfail(const char *fname)
{
	tlmp_error (MSG_R(E_CANTOPEN),fname,strerror(errno));
}


static int savefile_gen (_F_savefile &c, const char *fname, FILE *fout)
{
	c.priv = NULL;
	int ret = -1;
	if (fout == NULL){
		c.openfail (fname);
	}else{
		c.start(fout,fname);
		ret = c.dowrite (fout,fname);
		fflush (fout);
		c.end(fout,fname);
	}
	return ret;
}

int savefile (_F_savefile &c, const char *fname, bool append)
{
	FILE *fout = fopen(fname,append ? "a" : "w");
	int ret = savefile_gen (c,fname,fout);
	if (fout != NULL) if (fclose (fout) == -1) ret = -1;
	return ret;
}

int savefile (_F_savefile &c, CONFIG_FILE &cfg, bool append)
{
	FILE_CFG *fout = cfg.fopen(append ? "a" : "w");
	SSTRING tmp(cfg.getpath());
	int ret = savefile_gen (c,tmp.get(),filecfg_tofile(fout));
	if (fout != NULL) if (fclose (fout) == -1) ret = -1;
	return ret;
}

